@@ -47,7 +47,7 @@ mutation ForgeSubmit(
     $timezone: String,
     $groupArrayId: String,
     $groupArrayIndex: Int,
-    $errorType: String,
+    $webhookURL: String,
 ) {{
     forgeSubmit (
         forgeEid: $forgeEid,
@@ -60,7 +60,7 @@ mutation ForgeSubmit(
         timezone: $timezone,
         groupArrayId: $groupArrayId,
         groupArrayIndex: $groupArrayIndex,
-        errorType: $errorType
+        webhookURL: $webhookURL
     ) {query}
 }}
 """
